#include "stdafx.h"

#include "bitmap.h"
#include "ff3se.h"

//BMP header and info
BITMAPFILEHEADER	bmfh;
BITMAPINFOHEADER	bmih;
RGBQUAD				*aColors;
BYTE				*aBitmapBits;
	
//PCX header and info
PCXHEADER			pcxh;
RGBTRIPLE			tempPal[256];

//Standard constructor
Bitmap::Bitmap(void)
	{
	aColors = NULL;
	aBitmapBits = NULL;
	pixels = NULL;
	}

/*************************************/
//Standard destructor
Bitmap::~Bitmap(void)
	{
	SAFE_DELETE(pixels);
	DeleteObject(BMP);
	}

/*************************************/
//Adds the color c to the palette
//Has no effect if c is already in the palette
//Returns the palette entry of the color added, 257 if the Bitmap already has 256 colors
int Bitmap::addColor(RGBTRIPLE c)
	{
	int i;
	int found = -1;

		if(colorsUsed == 256)
			return 257;

	for(i = 0;(i < colorsUsed) && (found == -1);i++)
		{
		if((c.rgbtBlue == palette[i].rgbtBlue) && (c.rgbtGreen == palette[i].rgbtGreen) &&
		   (c.rgbtRed == palette[i].rgbtRed))
			found = i;
		}
	if(found == -1)
		{
		palette[colorsUsed].rgbtBlue = c.rgbtBlue;
		palette[colorsUsed].rgbtGreen = c.rgbtGreen;
		palette[colorsUsed].rgbtRed = c.rgbtRed;
		found = colorsUsed++;
		}
	
	return found;
	}

/*************************************/
//Sets pixel x,y to the color specified in RGBpixel
//Calls addColor to add the color to the palette entry 
//Returns the number of colors used by the Bitmap after adding the RGBpixel color
int Bitmap::convert256(RGBTRIPLE RGBpixel,int x,int y)
	{
	int found = addColor(RGBpixel);

	if(found == 257)
		return 257;
		
	pixels[width * y + x] = found;

	return colorsUsed;
	}

/*************************************/
//Copies the specified 8x8 tile passed as tile to the specified location in the Bitmap
//Returns 1 if the location is outside of Bitmap, 0 otherwise
BYTE Bitmap::copyTile(BYTE *tile,int x,int y)
	{
	BYTE *ptr;
	int x2,y2;
	if(((x * 8) >= (width - 7)) || ((y * 8) >= (height - 7)))
		return 1;

	ptr = &(pixels[x * 8 + (y * 8) * width]);
	for(y2 = 0; y2 < 8;y2++)
		{
		for(x2 = 0;x2 < 8;x2++)			
			{
			*(ptr++) = *(tile++);
			}
		ptr += width - 8;
		}
	
	return 0;
	}

/*************************************/
//Creates a blank bitmap of the specified width and height
//Returns 0 if succesful 1 otherwise
BYTE Bitmap::createBlank(int w,int h)
	{
	width = w;
	height = h;
	colorsUsed = 1;
	SAFE_DELETE(pixels);

	pixels = (BYTE *)malloc(w * h);
	if(!pixels)
		return 1;
	
	palette[0].rgbtBlue = 0;
	palette[0].rgbtGreen = 0;
	palette[0].rgbtRed = 0;

	ZeroMemory(pixels,(w * h));

	return 0;
	}

/*************************************/
//Returns the RGB color of the pixel at x,y
RGBTRIPLE Bitmap::getRGBPixel(int x,int y)
	{
	if((y >= height) || (x >= width))
		{
		RGBTRIPLE black;
		black.rgbtBlue = 0;
		black.rgbtGreen = 0;
		black.rgbtRed = 0;
		return black;
		}
	return palette[pixels[y * width + x]];
	}

/*************************************/
//Loads a bitmap from a filename
//Converts it to a 256 color paletted bitmap
//maxColors is the maximum number of colors the image can have
//maxWidth and maxHeight specify a maximum size for the image
//Returns 0 if successful, 1 if the image has over max colors, 2 if the image is too large,
//3 if an error occured, and 4 if it's not a valid BMP file
BYTE Bitmap::loadBMP(char *filename,int maxColors,int maxWidth,int maxHeight)
	{
	int handle;
	BYTE success;
	int i;

	handle = open(filename,O_RDWR | O_BINARY);

	if(handle == -1)
		return 3;

	SAFE_DELETE(aColors);
	SAFE_DELETE(aBitmapBits);
	SAFE_DELETE(pixels);

	read(handle,&bmfh,sizeof(bmfh));
	read(handle,&bmih,sizeof(bmih));

	if(bmfh.bfType != 19778)
		{
		close(handle);
		return 4;
		}

	width = bmih.biWidth;	
	height = bmih.biHeight;

	if((width > maxWidth) || (height > maxHeight))
		{
		close(handle);
		return 2;
		}

	pixels = (BYTE *)malloc(width * height);
	
	if(!pixels)
		{
		close(handle);
		return 3;
		}
	colorsUsed = 0;

	switch(bmih.biBitCount)
		{
		case 1:
			aColors = (RGBQUAD*) malloc(2 * sizeof(RGBQUAD));
			read(handle,aColors,2 * sizeof(RGBQUAD));						
		for(i = 0;i < 2;i++)
				{
				tempPal[i].rgbtBlue = aColors[i].rgbBlue;
				tempPal[i].rgbtGreen = aColors[i].rgbGreen;
				tempPal[i].rgbtRed = aColors[i].rgbRed;
				}
		break;
		case 4:
			aColors = (RGBQUAD*) malloc(16 * sizeof(RGBQUAD));
			read(handle,aColors,16 * sizeof(RGBQUAD));
			for(i = 0;i < 16;i++)
				{
				tempPal[i].rgbtBlue = aColors[i].rgbBlue;
				tempPal[i].rgbtGreen = aColors[i].rgbGreen;
				tempPal[i].rgbtRed = aColors[i].rgbRed;
				}
			break;
		case 8:
			aColors = (RGBQUAD*) malloc(256 * sizeof(RGBQUAD));
			read(handle,aColors,256 * sizeof(RGBQUAD));
			for(i = 0;i < 256;i++)
				{
				tempPal[i].rgbtBlue = aColors[i].rgbBlue;
				tempPal[i].rgbtGreen = aColors[i].rgbGreen;
				tempPal[i].rgbtRed = aColors[i].rgbRed;
				}
			break;
		case 16:
		case 24:
		case 32:
			aColors = NULL;
			break;
		}
	
	if((bmih.biBitCount < 16) && (aColors == NULL))	
		{
		SAFE_DELETE(aColors);
		close(handle);
		return 3;
		}

	aBitmapBits = (BYTE *)malloc(bmih.biSizeImage);
	
	if(!aBitmapBits)
		{
		SAFE_DELETE(aColors);
		close(handle);
		return 3;
		}

	read(handle,aBitmapBits,bmih.biSizeImage);
	
	close(handle);
	
	switch(bmih.biBitCount)
		{
		case 1:
			success = load1BitBMP(maxColors);
			break;
		case 4:
			success = load4BitBMP(maxColors);
			break;
		case 8:
			success = load8BitBMP(maxColors);
			break;
		case 16:
			success = load16BitBMP(maxColors);
			break;
		case 24:
			success = load24BitBMP(maxColors);
			break;
		case 32:
			success = load32BitBMP(maxColors);
			break;
		}
	
	SAFE_DELETE(aBitmapBits);
	SAFE_DELETE(aColors);

	return success;
	}

/*************************************/
//Loads data from a 1-bit bitmap
//Calls convert256 to convert it to 256 color format
//maxColors is the maximum number of colors the image can have
//Returns 0 if successful, 1 if the image has over max colors
BYTE Bitmap::load1BitBMP(int maxColors)
	{
	int lineSize;
	int x,y,i;

	BYTE c;

	int num;

	lineSize = (width / 8);				//aBitmapBit is zero padded to DWORD boundary
	if(lineSize % 4 != 0)				//Determine length of scanline
		do{lineSize++;} while(lineSize % 4 != 0);		//Ensure a DWORD aligned scanline

	for(y = 0;y < height;y++)
		{
		num = ((height - (y + 1)) * lineSize);
		for(x = 0;x < width;x++)
			{
			c = aBitmapBits[(num + (x / 8))];
			i = (7 - (x % 8));
			if((c & BIT(i)) > 0)
				{
				if(convert256(tempPal[1],x,y) > maxColors)
					return	1;
				}
			else
				{
				if(convert256(tempPal[0],x,y) > maxColors)
					return	1;
				}
			}
		}
	
	return 0;
	}

/*************************************/
//Loads data from a 4-bit bitmap
//Calls convert256 to convert it to 256 color format
//maxColors is the maximum number of colors the image can have
//Returns 0 if successful, 1 if the image has over max colors
BYTE Bitmap::load4BitBMP(int maxColors)
	{
	int lineSize;
	int x,y;
	BYTE num;
	int index;

	char c;

	//Copy 16 color palette
	for(int i = 0;i < 16;i++)
		palette[i] = tempPal[i];
	colorsUsed = 16;
	
	lineSize = (width / 2);				//aBitmapBit is zero padded to DWORD boundary
	if((lineSize % 4) != 0)				//Determine length of scanline
		do{lineSize++;} while(lineSize % 4 != 0);		//Ensure a DWORD aligned scanline

	for(y = 0;y < height;y++)
		{
		index = ((height - (y + 1)) * lineSize);
		for(x = 0;x < width;x++)
			{
			c = aBitmapBits[(index + (x / 2))];
			if((x % 2) != 0)
				{
				num = c & 0x0F;
				if(convert256(tempPal[num],x,y) > maxColors)
					return	1;
				}
			else
				{
				num = (c & 0xF0) >> 4;
				if(convert256(tempPal[num],x,y) > maxColors)
					return	1;
				}
			}
		}
	
	return 0;
	}

/*************************************/
//Loads data from a 8-bit bitmap
//Calls convert256 to strip unused colors
//maxColors is the maximum number of colors the image can have
//Returns 0 if successful, 1 if the image has over max colors
BYTE Bitmap::load8BitBMP(int maxColors)
	{
	int lineSize;
	int x,y;
	int num;

	BYTE *c;

	lineSize = width;					//aBitmapBit is zero padded to DWORD boundary
	if(lineSize %4 != 0)				//Determine length of scanline
		do{lineSize++;} while(lineSize % 4 != 0);		//Ensure a DWORD aligned scanline

	num = height * lineSize;

	for(y = 0;y < height;y++)
		{
		num -= lineSize;
		c = &(aBitmapBits[num]);
		for(x = 0;x < width;x++)
			{
			if(convert256(tempPal[*c],x,y) > maxColors)
				return	1;
			c++;
			}
		}
	
	return 0;
	}

/*************************************/
//Loads data from a 16-bit bitmap
//Calls convert256 to convert it to a 256 color bitmap
//maxColors is the maximum number of colors the image can have
//Returns 0 if successful, 1 if the image has over max colors
BYTE Bitmap::load16BitBMP(int maxColors)
	{
	int lineSize;
	int x,y;
	int num;
	BYTE c1,c2;

	RGBTRIPLE c;

	BYTE *ptr;
	
	lineSize = width * 2;				//aBitmapBit is zero padded to DWORD boundary
	if((lineSize % 4) != 0)				//Determine length of scanline
		do{lineSize++;} while(lineSize % 4 != 0);		//Ensure a DWORD aligned scanline

	for(y = 0;y < height;y++)
		{
		num = ((height - (y + 1)) * lineSize);
		for(x = 0;x < width;x++)
			{
			ptr = &(aBitmapBits[(num + (x * 2))]);
			c1 = *ptr++;
			c2 = *ptr++;
			c.rgbtBlue = (c1 & 0x1F) * 8;
			c.rgbtGreen = ((c1 & 0xE0) >> 5) + ((c2 & 0x03) << 3);
			c.rgbtGreen *= 8;
			c.rgbtRed = (c2 & 0x7C) << 1;
			if(convert256(c,x,y) > maxColors)
				return	1;
			}
		}
	
	return 0;
	}

/*************************************/
//Loads data from a 24-bit bitmap
//Calls convert256 to convert it to a 256 color bitmap
//maxColors is the maximum number of colors the image can have
//Returns 0 if successful, 1 if the image has over max colors
BYTE Bitmap::load24BitBMP(int maxColors)
	{
	int lineSize;
	int x,y;
	int num;

	RGBTRIPLE c;

	BYTE *ptr;
	
	lineSize = width * 3;				//aBitmapBit is zero padded to DWORD boundary
	if((lineSize % 4) != 0)				//Determine length of scanline
		do{lineSize++;} while(lineSize % 4 != 0);		//Ensure a DWORD aligned scanline

	for(y = 0;y < height;y++)
		{
		num = ((height - (y + 1)) * lineSize);
		for(x = 0;x < width;x++)
			{
			ptr = &(aBitmapBits[(num + (x * 3))]);
			c.rgbtBlue = *ptr++;
			c.rgbtGreen = *ptr++;
			c.rgbtRed = *ptr;
			if(convert256(c,x,y) > maxColors)
				return	1;
			}
		}
	
	return 0;
	}

/*************************************/
//Loads data from a 32-bit bitmap
//Calls convert256 to convert it to a 256 color bitmap
//maxColors is the maximum number of colors the image can have
//Returns 0 if successful, 1 if the image has over max colors
BYTE Bitmap::load32BitBMP(int maxColors)
	{
	int lineSize;
	int x,y;
	int num;

	RGBTRIPLE c;

	BYTE *ptr;
	
	lineSize = width * 4;

	for(y = 0;y < height;y++)
		{
		num = ((height - (y + 1)) * lineSize);
		for(x = 0;x < width;x++)
			{
			ptr = &(aBitmapBits[(num + (x * 4))]);
			c.rgbtBlue = *ptr++;
			c.rgbtGreen = *ptr++;
			c.rgbtRed = *ptr;
			ptr++;
			if(convert256(c,x,y) > maxColors)
				return	1;
			}
		}
	
	return 0;
	}

/*************************************/
//Loads a bitmap from a file named filename
//Determines the type of file by the extension and calls the appropriate load function
//Converts it to a 256 color paletted bitmap
//maxColors is the maximum number of colors the image can have
//maxWidth and maxHeight specify a maximum size for the image
//Returns 0 if successful, 1 if the image has over max colors, 2 if the image is too large,
//3 if an error occured, and 4 if it's not a valid image file
BYTE Bitmap::loadFile(char *filename,int maxColors,int maxWidth,int maxHeight)
	{
	BYTE success = 4;
	
	if((tolower(filename[strlen(filename) - 4]) == '.') &&
	   (tolower(filename[strlen(filename) - 3]) == 'b') &&
	   (tolower(filename[strlen(filename) - 2]) == 'm') &&
	   (tolower(filename[strlen(filename) - 1]) == 'p'))
		{
		success  = loadBMP(filename,maxColors,maxWidth,maxHeight);
		}
	if((tolower(filename[strlen(filename) - 4]) == '.') &&
	   (tolower(filename[strlen(filename) - 3]) == 'p') &&
	   (tolower(filename[strlen(filename) - 2]) == 'c') &&
	   (tolower(filename[strlen(filename) - 1]) == 'x'))
		{
		success = loadPCX(filename,maxColors,maxWidth,maxHeight);
		}
		
	if((tolower(filename[strlen(filename) - 4]) == '.') &&
	   (tolower(filename[strlen(filename) - 3]) == 'p') &&
	   (tolower(filename[strlen(filename) - 2]) == 'n') &&
	   (tolower(filename[strlen(filename) - 1]) == 'g'))
		{
		success = loadPNG(filename,maxColors,maxWidth,maxHeight);
		}

	return success;
	}

/*************************************/
//Saves a bitmap to a file named filename
//Determines the type of file by the extension and calls the appropriate save function
//Return of 0 = success, 1 = fvailure, 2 = invalid filename
BYTE Bitmap::saveFile(char *filename,int maxX,int maxY)
	{
	BYTE success = 2;
	
	if((tolower(filename[strlen(filename) - 4]) == '.') &&
	   (tolower(filename[strlen(filename) - 3]) == 'b') &&
	   (tolower(filename[strlen(filename) - 2]) == 'm') &&
	   (tolower(filename[strlen(filename) - 1]) == 'p'))
		{
		success  = saveBMP(filename,maxX,maxY);
		}
	if((tolower(filename[strlen(filename) - 4]) == '.') &&
	   (tolower(filename[strlen(filename) - 3]) == 'p') &&
	   (tolower(filename[strlen(filename) - 2]) == 'c') &&
	   (tolower(filename[strlen(filename) - 1]) == 'x'))
		{
		success = savePCX(filename,maxX,maxY);
		}
		
	if((tolower(filename[strlen(filename) - 4]) == '.') &&
	   (tolower(filename[strlen(filename) - 3]) == 'p') &&
	   (tolower(filename[strlen(filename) - 2]) == 'n') &&
	   (tolower(filename[strlen(filename) - 1]) == 'g'))
		{
		success = savePNG(filename,maxX,maxY);
		}

	return success;
	}

/*************************************/
//Saves the Bitmap as a .BMP file
BYTE Bitmap::saveBMP(char *filename,int maxX,int maxY)
	{
	int handle;
	BYTE *ptr;
	BYTE *ptr2;
	int x,y;
	BYTE zero = 0;

	handle = open(filename,O_WRONLY | O_BINARY | O_CREAT | O_TRUNC, S_IWRITE);
	
	if(handle == -1)
		return 1;

	bmfh.bfType = 19778;
	//bmfh.bfSize = maxX * maxY * 3 + sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER);
	bmfh.bfSize = (maxX * maxY) / 2 + sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + 64;
	bmfh.bfReserved1 = 0;
	bmfh.bfReserved2 = 0;
	bmfh.bfOffBits = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + 64;

	write(handle,&bmfh,sizeof(bmfh));

	bmih.biSize = sizeof(BITMAPINFOHEADER);
	bmih.biWidth = maxX;
	bmih.biHeight = maxY;
	bmih.biPlanes = 1;
	//bmih.biBitCount = 24;
	bmih.biBitCount = 4;
	bmih.biCompression = BI_RGB;
	//bmih.biSizeImage = maxX * maxY * 3;
	bmih.biSizeImage = (maxX * maxY) / 2;
	bmih.biXPelsPerMeter  = 4000;
	bmih.biYPelsPerMeter = 4000;
	bmih.biClrUsed = 0;
	bmih.biClrImportant = 0;

	write(handle,&bmih,sizeof(bmih));

	//Write palette info
	for(int i =0;i < 16;i++)
		{
		write(handle,&(palette[i].rgbtBlue),1);
		write(handle,&(palette[i].rgbtGreen),1);
		write(handle,&(palette[i].rgbtRed),1);
		write(handle,&zero,1);
		}

//	aBitmapBits = (BYTE *)malloc(maxX * maxY * 3);
	aBitmapBits = (BYTE *)malloc((maxX * maxY) / 2);

	ptr = aBitmapBits;
	
	ptr2 = pixels + (maxY - 1) * width;

	for(y = 0;y < maxY;y++)
		{
		for(x = 0;x < maxX;x += 2)
			{
//			*(ptr++) = palette[*ptr2].rgbtBlue;
//			*(ptr++) = palette[*ptr2].rgbtGreen;
//			*(ptr++) = palette[*ptr2++].rgbtRed;
			*ptr = *(ptr2++) * 16;
			*(ptr++) += *(ptr2++);
			}
		ptr2 += (width - maxX);
		ptr2 -= width * 2;
		}
	
//	write(handle,aBitmapBits,maxX * maxY * 3);
	write(handle,aBitmapBits,(maxX * maxY) / 2);

	SAFE_DELETE(aBitmapBits);
	close(handle);

	return 0;
	}

/*************************************/
//Returns the number of colors used by the image
BYTE Bitmap::numColors(void)
	{
	return colorsUsed;
	}

/*************************************/
//Sets palette entry i to color c
//If chek is true, Returns 1 if i is greater than or equal to colorsUsed
//otherwise sets colorsUsed equal to i + 1
BYTE Bitmap::setColor(BYTE i,RGBTRIPLE c,bool check)
	{
	if(i >= colorsUsed)
		{
		if(check)
			return 1;
		else
			colorsUsed = i + 1;
		}
	palette[i].rgbtBlue = c.rgbtBlue;
	palette[i].rgbtGreen = c.rgbtGreen;
	palette[i].rgbtRed = c.rgbtRed;
	
	return 0;
	}

/*************************************/
//Sets the pixel at x,y to the RGB color of C
//If that color is not in the palette it is added
//Returns 1 if the color is not in the palette and the max colors is already used, 0 otherwise
BYTE Bitmap::setRGBPixel(int x,int y,RGBTRIPLE c)
	{
	int found;

	found = addColor(c);

	if(found == 257)
		return 1;
	
	pixels[y * width + x] = found;
	return 0;
	}

/*************************************/
//Determines which color should be transparent and make that color palette entry 0
//The color used most is set to transparent
void Bitmap::setTransparency(void)
	{
	int x,y;
	int usage[256];
	BYTE *ptr;
	int i;
	BYTE transColor;
	int maxUsed;

	ZeroMemory(&usage,sizeof(usage));
	
	ptr = pixels;
	
	for(y = 0;y < height;y++)		//Determine amount each color is used
		{
		for(x = 0;x < width;x++)
			{
			usage[(*ptr++)]++;
			}
		}
	
	transColor = 0;
	maxUsed = 0;
	for(i = 0;i < colorsUsed;i++)	//Determine which color is used most
		{
		if(usage[i] > maxUsed)
			{
			maxUsed = usage[i];
			transColor = i;			
			}
		}
	
	if(transColor != 0)
		swapPal(0,transColor);		//Set that color to palette entry 0
	}

/*************************************/
//Makes all pixels of color A become color B and vice versa
void Bitmap::swapPal(int a,int b)
	{
	int x,y;
	RGBTRIPLE temp;
	BYTE *ptr;

	if(a == b)		//No need to swap a palette entry with itself
		return;

	memcpy(&temp,&(palette[a]),sizeof(RGBTRIPLE));
	memcpy(&(palette[a]),&(palette[b]),sizeof(RGBTRIPLE));
	memcpy(&(palette[b]),&temp,sizeof(RGBTRIPLE));

	ptr = pixels;
	
	for(y = 0;y < height;y++)
		{
		for(x = 0;x < width;x++)
			{
			if(*ptr == a)
				{
				*ptr= b;
				}
			else  if(*ptr == b)
				{
				*ptr = a;
				}
			ptr++;
			}
		}
	}

/*************************************/
//Draw the bitmap to the DC passed in
//backBmp is the image to use as the transparent background
//w and h are the width and height of the background image
//if backBmp is NULL the image will not be drawn transparently
void Bitmap::drawToDC(HDC drawDC,Bitmap* backBmp,int w,int h)
	{
	if(w == 0)
		w = width;
	if(h == 0)
		h = height;

	if(backBmp != NULL)
		{
		Bitmap newBmp;
		int x,y;
		int i;

		//Create composite Bitmap with backBmp as the background with the Bitmap on top of it
		newBmp.createBlank(w,h);
		
		for(i = 0;i < colorsUsed;i++)
			{
			newBmp.setColor(i,palette[i]);
			}
		for(i = 0;i < backBmp->colorsUsed;i++)
			{
			newBmp.setColor(i + colorsUsed,backBmp->palette[i]);
			}

		BYTE *ptr;
		BYTE *ptr2;

		for(y = 0;y < h;y++)
			{
			ptr = &(pixels[y * width]);
			ptr2 = &(backBmp->pixels[y * backBmp->width]);
			for(x = 0;x < w;x++)
				{
				if(getPixel(x,y) == 0)
					newBmp.setPixel(x,y,*ptr2 + colorsUsed);
				else
					newBmp.setPixel(x,y,*ptr);
				ptr++;
				ptr2++;
				}
			}
		newBmp.drawToDC(drawDC);		//Draw the new composite Bitmap to the DC
		}
	else
		{
		BITMAPINFO *bmi;
		HDC memHdc;
		int i;

		bmi = (BITMAPINFO*)malloc(sizeof(BITMAPINFO) + 4 * (colorsUsed - 1));

		DeleteObject(BMP);

		memHdc = CreateCompatibleDC(screenHdc);

		bmi->bmiHeader.biBitCount = 8;
		bmi->bmiHeader.biClrImportant = colorsUsed;
		bmi->bmiHeader.biClrUsed = colorsUsed;
		bmi->bmiHeader.biCompression = 0;
		bmi->bmiHeader.biHeight = -1 * height;
		bmi->bmiHeader.biPlanes = 1;
		bmi->bmiHeader.biSize = sizeof(bmi->bmiHeader);
		bmi->bmiHeader.biSizeImage = 0;
		bmi->bmiHeader.biWidth = width;
	
		for(i = 0;i < colorsUsed;i++)
			{
			bmi->bmiColors[i].rgbBlue = palette[i].rgbtBlue;
			bmi->bmiColors[i].rgbGreen = palette[i].rgbtGreen;
			bmi->bmiColors[i].rgbRed = palette[i].rgbtRed;
			bmi->bmiColors[i].rgbReserved = 0;
			}

		BMP = CreateDIBitmap(screenHdc,(BITMAPINFOHEADER *)bmi,CBM_INIT,(VOID *)pixels,(BITMAPINFO *)bmi,DIB_RGB_COLORS);
		SelectObject(memHdc,BMP);
		BitBlt(drawDC,0,0,w,h,memHdc,0,0,SRCCOPY);

		DeleteDC(memHdc);
		SAFE_DELETE(bmi);
		}
	}

/*************************************/
//Draw the bitmap to the DC passed in, multiplys image size by mult
//backBmp is the image to use as the transparent background
//w and h are the width and height of the background image
//if backBmp is NULL the image will not be drawn transparently
//Creates a new enlarged image Bitmap, then calls drawToDC to draw it to the screen
void Bitmap::drawToLargeDC(BYTE mult,HDC drawDC,int srcx,int srcy,Bitmap *backBmp,int w,int h)
	{
	int x,y;
	int x2,y2;
	Bitmap newBmp;
	BYTE c;
	int i;

	if(w == 0)
		w = width * mult;
	if(h == 0)
		h = height * mult;
			
	newBmp.createBlank(w,h);

	for(i = 0;i < colorsUsed;i++)
		{
		newBmp.setColor(i,palette[i]);
		}
	if(backBmp != NULL)
		{
		for(i = 0;i < backBmp->colorsUsed;i++)
			{
			newBmp.setColor(i + colorsUsed,backBmp->palette[i]);
			}
		}
	for(i = 0;i < colorsUsed;i++)
		{
		newBmp.setColor(i,palette[i]);
		}
	for(y = 0;y < h;y++)
		{
		y2 = (y / mult) + srcy;
		for(x = 0;x < w;x ++)
			{
			x2 = (x / mult) + srcx;
			c = getPixel(x2,y2);
			if((c == 0) && (backBmp != NULL))
				{
				c = backBmp->getPixel(x,y) + colorsUsed;
				}
			newBmp.setPixel(x,y,c);
			}
		}
	
	newBmp.drawToDC(drawDC,NULL,w,h);
	}

/*************************************/
//Creates the Bitmap from a Window's device context
void Bitmap::createFromDC(HDC tempDC,int w,int h)
	{
	int x,y;

	createBlank(w,h);

	for(y = 0;y < h;y++)
		{
		for(x = 0;x < w;x++)
			{
//			r = COLORREFtoRGB(GetPixel(tempDC,x,y));
//			setRGBPixel(x,y,r);
			setRGBPixel(x,y,COLORREFtoRGB(GetPixel(tempDC,x,y)));
			}
		}
	}

/*************************************/
//Flips the Bitmap
//If flip is 0, flips it horizontally, else flips if vetically
//If onlyFlipUsed is set, only slips the portion of the Bitmap that is used
void Bitmap::flipImage(int flip,bool onlyFlipUsed)
	{
	Bitmap temp;
	BYTE *ptr;
	int x,y;
	int maxX = 0,maxY = 0;

	if(onlyFlipUsed)
		{
		ptr = pixels;
		for(y = 0;y < height;y++)
			{
			for(x = 0;x < width;x++)
				{
				if(*(ptr++) > 0)
					{
					if(x > maxX)
						maxX = x;
					if(y > maxY)
						maxY = y;
					}
				}
			}
		maxX++;
		maxY++;
		}
	else
		{
		maxY = height;
		maxX = width;
		}

	temp.createBlank(width,height);

	memcpy(temp.pixels,pixels,width * height);

	ptr = pixels;

	for(y = 0;y < maxY;y++)
		{		
		for(x = 0;x < maxX;x++)
			{
			if(flip == 0)
				*(ptr++) = temp.pixels[(maxX - x - 1) + y * width];
			else
				*(ptr++) = temp.pixels[x + (maxY - y - 1) * width];
			}
		ptr += (width - maxX);
		}
	}

/*************************************/
//Removes all rows and columns of all 0's from the top and left of the Bitmap
void Bitmap::corner(void)
	{
	bool done;
	int x,y;
	BYTE *ptr;
	int i;
	int numZeros = 0;

	//Exit if the image is entirely 0's
	ptr = pixels;
	for(y = 0;y < height;y++)
		{
		for(x = 0;x < width;x++)
			{
			if(*(ptr++) == 0)
				numZeros++;
			}
		}
	if(numZeros == (width * height))
		return;

	//remove column on the left
	do {
		done = false;
		
		//Check to see if the colum,n is all 0's
		for(y = 0;y < height;y++)
			{
			if(pixels[y * width] != 0)
				{
				done = true;
				break;
				}
			}
		//If it is then shift Bitmap left 1 pixel
		if(!done)
			{
			ptr = pixels;
			for(i = 0;i < (width * height - 1);i++)
				{
				*(ptr) = *(ptr + 1);
				ptr++;
				}
			*ptr = 0;
			}
	} while(!done);		//Check next column

	//remove row on top
	do {
		done = false;
		
		//Check to see if the colum,n is all 0's
		for(x = 0;x < width;x++)
			{
			if(pixels[x] != 0)
				{
				done = true;
				break;
				}
			}
		//If it is then shift Bitmap up 1 pixel
		if(!done)
			{
			ptr = pixels;
			for(i = 0;i < (width * (height - 1));i++)
				{
				*(ptr) = *(ptr + width);
				ptr++;
				}
			for(x = 0;x < width;x++)
				*(ptr++) = 0;
			}
	} while(!done);		//Check next row
	}
