#include "stdafx.h"

#include "bitmap.h"
#include "ff3se.h"

/*************************************/
//Loads a bitmap from filename (PCX file)
//Converts it to a 256 color paletted bitmap
//maxColors is the maximum number of colors the image can have
//maxWidth and maxHeight specify a maximum size for the image
//Returns 0 if successful, 1 if the image has over max colors, 2 if the image is too large,
//3 if an error occured, and 4 if it's not a valid PCX file
BYTE Bitmap::loadPCX(char *filename,int maxColors,int maxWidth,int maxHeight)
	{
	int handle;
	BYTE success;

	handle = open(filename,O_RDWR | O_BINARY);

	if(handle == -1)
		return 3;

	SAFE_DELETE(aColors);
	SAFE_DELETE(aBitmapBits);
	SAFE_DELETE(pixels);
	
	read(handle,&pcxh,sizeof(PCXHEADER));

	if(pcxh.ID != 10)		//Check to make sure it's a valid PCX file
		{
		close(handle);
		return 4;
		}

	width = pcxh.xmax - pcxh.xmin + 1;	
	height = pcxh.ymax - pcxh.ymin + 1;

	if((width > maxWidth) || (height > maxHeight))		//Return if larger than max size
		{
		close(handle);
		return 2;
		}

	colorsUsed = 0;

	switch(pcxh.nplanes)
		{
		case 1:
			switch(pcxh.bpp)
				{
				case 1:
					success = load1BitPCX(maxColors,handle);
					break;
				case 4:
					success = load4BitPCX(maxColors,handle);
					break;
				case 8:
					success = load8BitPCX(maxColors,handle);
					break;	
				break;
				}
			break;
		case 3:
			success = load24BitPCX(maxColors,handle);
			break;
		}
	
	close(handle);

	return success;
	}

/*************************************/
//Loads data from a 1-bit PCX file
//Calls convert256 to convert it to a 256 color bitmap
//maxColors is the maximum number of colors the image can have
//handle is the file toread from
//Returns 0 if successful, 1 if the image has over max colors
BYTE Bitmap::load1BitPCX(int maxColors,int handle)
	{
	int x,y,i,j;
	BYTE c;
	int num;
	BYTE *ptr;
	long size;
	RGBTRIPLE *RGBpixel;

	RGBpixel = (RGBTRIPLE *)malloc(pcxh.bytesPerLine * sizeof(RGBTRIPLE));

	lseek(handle,0,SEEK_END);

	size = tell(handle) - sizeof(PCXHEADER);	//Determine size of compressed data
	aBitmapBits = (BYTE *)malloc(size);

	lseek(handle,sizeof(PCXHEADER),SEEK_SET);
	
	read(handle,aBitmapBits,size);

	pixels = (BYTE *)malloc(width * height);

	ptr = aBitmapBits;

	tempPal[0].rgbtRed = 0;
	tempPal[0].rgbtGreen = 0;
	tempPal[0].rgbtBlue = 0;
	tempPal[1].rgbtRed = 255;
	tempPal[1].rgbtGreen = 255;
	tempPal[1].rgbtBlue = 255;

	for(y = 0;y < height;y++)
		{
		x = 0;
		do {
			c = *(ptr++);
			if((c & 0xC0) != 0xC0)
				{
				for(j = 0;j < 8;j++)
					{
					if(x < width)
						if(convert256(tempPal[(c & BIT(7 - j)) >> (7 - j)],x,y) > maxColors)
							return	1;
					x++;
					}
				}
			else
				{
				num = c & 0x3F;
				c = *(ptr++);
				for(i = 0;i < num;i++)
					{
					for(j = 0;j < 8;j++)
						{
						if(x < width)
							if(convert256(tempPal[(c & BIT(7 - j)) >> (7 - j)],x,y) > maxColors)
								return	1;
						x++;
						}
					}
				}
		} while(x < (pcxh.bytesPerLine * 8));
		if((width %2 ) != 0)
			ptr++;
		}
	
	SAFE_DELETE(aBitmapBits);

	return 0;
	}

/*************************************/
//Loads data from a 4-bit PCX file
//Calls convert256 to convert it to a 256 color bitmap
//maxColors is the maximum number of colors the image can have
//handle is the file toread from
//Returns 0 if successful, 1 if the image has over max colors
BYTE Bitmap::load4BitPCX(int maxColors,int handle)
	{
	int x,y,i;
	BYTE c;
	int num;
	BYTE *ptr;
	long size;
	RGBTRIPLE *RGBpixel;

	RGBpixel = (RGBTRIPLE *)malloc(pcxh.bytesPerLine * sizeof(RGBTRIPLE));

	lseek(handle,0,SEEK_END);

	size = tell(handle) - sizeof(PCXHEADER);	//Determine size of compressed data
	aBitmapBits = (BYTE *)malloc(size);

	lseek(handle,sizeof(PCXHEADER),SEEK_SET);
	
	read(handle,aBitmapBits,size);

	pixels = (BYTE *)malloc(width * height);

	ptr = aBitmapBits;

	for(i = 0;i < 16;i++)
		{
		tempPal[i].rgbtRed = pcxh.colorMap[i * 3];
		tempPal[i].rgbtGreen = pcxh.colorMap[i * 3 + 1];
		tempPal[i].rgbtBlue = pcxh.colorMap[i * 3 + 2];
		}

	//Copy 16 color palette
	for(i = 0;i < 16;i++)
		palette[i] = tempPal[i];
	colorsUsed = 16;
	
	for(y = 0;y < height;y++)
		{
		x = 0;
		do {
			c = *(ptr++);
			if((c & 0xC0) != 0xC0)
				{
				if(convert256(tempPal[(c & 0xF0) >> 4],x,y) > maxColors)
					return	1;
				x++;
				if(x != width)
					{
					if(convert256(tempPal[c & 0x0F],x,y) > maxColors)
						return	1;
					}
				x++;
				}
			else
				{
				num = c & 0x3F;
				c = *(ptr++);
				for(i = 0;i < num;i++)
					{
					if(convert256(tempPal[(c & 0xF0) >> 4],x,y) > maxColors)
						return	1;
					x++;
					if(x != width)
						{
						if(convert256(tempPal[c & 0x0F],x,y) > maxColors)
							return	1;
						}
					x++;
					}
				}
		} while(x < (pcxh.bytesPerLine * 2));
		if((width %2 ) != 0)
			ptr++;
		}
	
	SAFE_DELETE(aBitmapBits);

	return 0;
	}
	
/*************************************/
//Loads data from a 8-bit PCX file
//Calls convert256 to strip[ unused colors
//maxColors is the maximum number of colors the image can have
//handle is the file toread from
//Returns 0 if successful, 1 if the image has over max colors
BYTE Bitmap::load8BitPCX(int maxColors,int handle)
	{
	int x,y;
	BYTE c;
	int num,i;
	BYTE *ptr;
	long size;

	lseek(handle,-768,SEEK_END);

	size = tell(handle) - sizeof(PCXHEADER);		//Determine size of compressed data
	aBitmapBits = (BYTE *)malloc(size);

	read(handle,&tempPal,768);						//Read the palette

	for(i = 0;i < 256;i++)
		{
		BYTE temp;

		temp =tempPal[i].rgbtBlue;
		tempPal[i].rgbtBlue = tempPal[i].rgbtRed;
		tempPal[i].rgbtRed = temp;
		}

	lseek(handle,sizeof(PCXHEADER),SEEK_SET);
	
	read(handle,aBitmapBits,size);

	pixels = (BYTE *)malloc(width * height);

	ptr = aBitmapBits;

	for(y = 0;y < height;y++)
		{
		x = 0;
		do {
			c = *(ptr++);
			if((c & 0xC0) != 0xC0)
				{
				if(convert256(tempPal[c],x,y) > maxColors)
					return	1;
				x++;
				}
			else
				{
				num = c & 0x3F;
				c = *(ptr++);
				for(i = 0;i < num;i++)
					{
					if(convert256(tempPal[c],x,y) > maxColors)
						return	1;
					x++;					
					}
				}
		} while(x < pcxh.bytesPerLine);
		if((width %2 ) != 0)
			ptr++;
		}
	
	SAFE_DELETE(aBitmapBits);

	return 0;
	}
	
/*************************************/
//Loads data from a 24-bit PCX file
//Calls convert256 to convert it to a 256 color bitmap
//maxColors is the maximum number of colors the image can have
//handle is the file toread from
//Returns 0 if successful, 1 if the image has over max colors
BYTE Bitmap::load24BitPCX(int maxColors,int handle)
	{
	int x,y,i;
	BYTE c;
	int num,p;
	BYTE *ptr;
	long size;
	RGBTRIPLE *RGBpixel;

	RGBpixel = (RGBTRIPLE *)malloc(pcxh.bytesPerLine * sizeof(RGBTRIPLE));

	lseek(handle,0,SEEK_END);

	size = tell(handle) - sizeof(PCXHEADER);	//Determine size of compressed data
	aBitmapBits = (BYTE *)malloc(size);

	lseek(handle,sizeof(PCXHEADER),SEEK_SET);
	
	read(handle,aBitmapBits,size);

	pixels = (BYTE *)malloc(width * height);

	ptr = aBitmapBits;

	for(y = 0;y < height;y++)
		{
		for(p = 0;p < 3;p++)
			{
			x = 0;
			do {
				c = *(ptr++);
				if((c & 0xC0) != 0xC0)
					{
					switch(p)
						{
						case 0:
							RGBpixel[x].rgbtRed = c;
							break;
						case 1:
							RGBpixel[x].rgbtGreen = c;
							break;
						case 2:
							RGBpixel[x].rgbtBlue = c;
							break;
						}
					x++;
					}
				else
					{
					num = c & 0x3F;
					c = *(ptr++);
					for(i = 0;i < num;i++)
						{
						switch(p)
							{
							case 0:
								RGBpixel[x].rgbtRed = c;
								break;
							case 1:
								RGBpixel[x].rgbtGreen = c;
								break;
							case 2:
								RGBpixel[x].rgbtBlue = c;
								break;
							}
						x++;					
						}
					}
			} while(x < pcxh.bytesPerLine);
			if((width %2 ) != 0)
				ptr++;
			}
		
		for(x = 0;x < width;x++)
			{
			if(convert256(RGBpixel[x],x,y) > maxColors)
				return	1;
			}
		}
	
	SAFE_DELETE(aBitmapBits);
	SAFE_DELETE(RGBpixel);

	return 0;
	}

/*************************************/
//Copresses a line into RLE format
//Returns length of line
int compressLine(BYTE *line,int len)
	{
	BYTE tempLine[256];
	int i = 0;
	int j = 0;
	int num;

	do {
		num = 0;
		do {
			i++;
			num++;	
		} while((num < 63) && (line[i - 1] == line[i]) && (i < len));
	
		if(num == 1)
			if(line[i - 1] < 192)
				tempLine[j++] = line[i - 1];
			else
				{
				tempLine[j++] = 193;
				tempLine[j++] = line[i - 1];
				}
		else
			{
			tempLine[j++] = num  + 192;
			tempLine[j++] = line[i - 1];
			}
	} while(i < len);
		
	memcpy((char *)line,(char *)tempLine,j);

	return j;
	}

/*************************************/
//Saves the Bitmap as a .PCX file
BYTE Bitmap::savePCX(char *filename,int maxX,int maxY)
	{
	int handle;
	BYTE line[256];
	int x,y;
	int len;
	int i;

	handle = open(filename,O_WRONLY | O_BINARY | O_CREAT | O_TRUNC,S_IWRITE);

	if(handle == -1)
		return 1;

	pcxh.ID = 10;
	pcxh.version = 5;
	pcxh.encoding = 1;
//	pcxh.bpp = 8;
	pcxh.bpp = 4;
	pcxh.xmin = 0;
	pcxh.ymin = 0;
	pcxh.xmax = maxX - 1;
	pcxh.ymax = maxY - 1;
	pcxh.hres = 1024;
	pcxh.vres = 768;
//	pcxh.nplanes = 3;
	pcxh.nplanes = 1;
//	pcxh.bytesPerLine = maxX;
	pcxh.bytesPerLine = maxX / 2;
	pcxh.palType = 1;
	
	//Set palette info
	for(i = 0;i < 16;i++)
		{
		pcxh.colorMap[i * 3] = palette[i].rgbtRed;
		pcxh.colorMap[(i * 3) + 1] = palette[i].rgbtGreen;
		pcxh.colorMap[(i * 3) + 2] = palette[i].rgbtBlue;
		}	

	write(handle,&pcxh,sizeof(pcxh));

	for(y = 0;y < maxY;y++)
		{
/*		for(x = 0;x < maxX;x++)
			{
			line[x] = palette[pixels[x + y * width]].rgbtRed;
			}
			len  = compressLine(line,maxX);
			write(handle,&line,len);
		for(x = 0;x < maxX;x++)
			{
			line[x] = palette[pixels[x + y * width]].rgbtGreen;
			}
			len  = compressLine(line,maxX);
			write(handle,&line,len);
		for(x = 0;x < maxX;x++)
			{
			line[x] = palette[pixels[x + y * width]].rgbtBlue;
			}
			len  = compressLine(line,maxX);
			write(handle,&line,len);*/
		for(x = 0;x < maxX;x += 2)
			{
			line[x / 2] = pixels[x + y * width] * 16 + pixels[(x + 1) + (y * width)];
			}
			len  = compressLine(line,maxX / 2);
			write(handle,&line,len);
		}
	
	close(handle);

	return 0;
	}
